@@ -27,8 +27,12 @@ from calculate.lib.utils.portage import isPkgInstalled
 from calculate.lib.utils.common import cmpVersion
 from itertools import dropwhile
 import re
+import contextlib
+import multiprocessing
+from collections import namedtuple
 
 from calculate.lib.cl_lang import setLocalTranslate
+
 setLocalTranslate('cl_lib3',sys.modules[__name__])
 
 _envData = [('system', '/etc/calculate/calculate.env'),
@@ -131,6 +135,11 @@ class VariableClPassStep(Variable):
     Pass for templates join 1,2,3,4,5 and etc
     """
 
+class VariableClTaskName(ReadonlyVariable):
+    """
+    Название задачи выполняемой cl-core
+    """
+
 class VariableClPassFile(Variable):
     """
     Template file performed at now
@@ -209,6 +218,7 @@ class VariableClAction(ReadonlyVariable):
     Example: install, uninstall, merge, domain, undomain, system, desktop
     """
 
+
 class VariableClPassState(ReadonlyVariable):
     """
     Program state
@@ -260,13 +270,18 @@ class VariableClProtectUseSet(ReadonlyVariable):
     def get(self):
         return "on" if self.Get('cl_human_edit_set') == 'off' else "off"
 
+class VariableClDispatchConfDefault(Variable):
+    """
+    Используется для обхода экспертных параметров
+    """
+    value = "dispatch"
+
 class VariableClDispatchConf(Variable):
     """
 
     """
     type = "choice"
     opt = ["--conf"]
-    value = "dispatch"
     syntax = "--{choice}-conf"
     metavalue = "METHOD"
 
@@ -276,6 +291,9 @@ class VariableClDispatchConf(Variable):
                     ",\n'dispatch' - " + _("manually update config files")
         self.label = _("Method for updating config files")
 
+    def get(self):
+        return self.Get('cl_dispatch_conf_default')
+
     def choice(self):
         return [("usenew",_("Use the new config files")),
                 ("skip",_("Skip the update of config files")),
@@ -393,17 +411,47 @@ class VariableClPkgdir(ReadonlyVariable):
     """
     PKGDIR
     """
+
     def get(self):
         emerge_config = self.Get('cl_emerge_config')
         if emerge_config and "PKGDIR" in emerge_config:
             return emerge_config['PKGDIR'].encode('utf-8')
         else:
             emergeInfo = self.Get('cl_emerge_info')
-            for line in filter(lambda x:x.startswith("PKGDIR="),
+            for line in filter(lambda x: x.startswith("PKGDIR="),
                                emergeInfo):
                 return line.partition("=")[2].strip('\n"\'')
         return ""
 
+class VariableClFeatures(ReadonlyVariable):
+    """
+    PKGDIR
+    """
+    type = "list"
+
+    def get(self):
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config and "FEATURES" in emerge_config:
+            return filter(
+                None, emerge_config['FEATURES'].encode('utf-8').split())
+        else:
+            emergeInfo = self.Get('cl_emerge_info')
+            for line in filter(lambda x: x.startswith("FEATURES="),
+                               emergeInfo):
+                return filter(
+                    None, line.partition("=")[2].strip('\n"\'').split())
+        return ""
+
+class VariableClDistdir(ReadonlyVariable):
+    """
+    DISTDIR
+    """
+    def get(self):
+        emerge_config = self.Get('cl_emerge_config')
+        if emerge_config and "DISTDIR" in emerge_config:
+            return emerge_config['DISTDIR'].encode('utf-8')
+        return "/var/calculate/remote/distfiles"
+
 class VariableClRepositoryData(ReadonlyTableVariable):
     """
     """
@@ -480,20 +528,80 @@ class VariableClPortdirOverlay(ReadonlyVariable):
                            where='cl_repository_name',
                            ne="gentoo") or []
 
+
+RepoConfigProxy = namedtuple("RepoConfigProxy", ["name", "location"])
+
+@contextlib.contextmanager
+def stderr_devnull():
+    oldstderr = sys.stderr
+    sys.stderr = open(os.devnull,'w')
+    try:
+        yield
+    finally:
+        sys.stderr = oldstderr
+
+class PortageConfigProxy(object):
+    def __init__(self):
+        self._dict = {}
+        self.repositories = []
+
+    def __getitem__(self, item):
+        return self._dict[item]
+
+    def __setitem__(self, key, value):
+        self._dict[key] = value
+
+    def __contains__(self, item):
+        return item in self._dict
+
+    @classmethod
+    def getconfig(cls, chroot):
+        from portage.package.ebuild.config import config
+
+        def target(chroot_path, queue):
+            try:
+                os.chroot(chroot_path)
+                with stderr_devnull():
+                    c = config()
+                obj = cls()
+                for key in ('CONFIG_PROTECT_MASK', 'CONFIG_PROTECT',
+                            'FEATURES',
+                            'PKGDIR', 'EMERGE_DEFAULT_OPTS', 'LINGUAS'):
+                    obj[key] = c[key]
+                obj.repositories = [RepoConfigProxy(x.name, x.location)
+                                    for x in c.repositories]
+                queue.put(obj)
+            except Exception as e:
+                queue.put(None)
+        queue = multiprocessing.Queue()
+        p = multiprocessing.Process(target=target, args=(chroot, queue))
+        p.start()
+        p.join(10)
+        return queue.get(timeout=10)
+
+    def __str__(self):
+        return "EmergeConfig"
+
 class VariableClEmergeConfig(ReadonlyVariable):
     """
     Emerge config object
     """
     type = "object"
+    systemRoot = "/"
 
     def get(self):
         try:
-            from portage.package.ebuild.config import config
-            config.__str__ = lambda self: "EmergeConfig"
-            c = config()
-            if c.repositories.repoLocationList() and c["LINGUAS"]:
-                return c
-        except:
+            if self.systemRoot == '/':
+                from portage.package.ebuild.config import config
+                config.__str__ = lambda self: "EmergeConfig"
+                with stderr_devnull():
+                    c = config()
+                if c.repositories.repoLocationList(): # and c["LINGUAS"]:
+                    return c
+            else:
+                c = PortageConfigProxy.getconfig(self.systemRoot)
+                return c or ""
+        except Exception as e:
             pass
         return ""
 
